/*
 * Copyright (c) 2004-2005 Northwestern University.
 *                         NuCAD Group.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

/*
 * acg_racg.h - Algorithms for ACG and Reduced ACG.
 *
 * Authors: Hai Zhou, haizhou@ece.northwestern.edu
 *          Jia Wang, jwa112@ece.northwestern.edu
 */

#ifndef ACG_RACG_H
#define ACG_RACG_H

#include "acg_base.h"

namespace acg
{

// direct build
void racg_build(acg_base_t &base);

// head/tail insert/erase
void racg_push(acg_base_t &base,
	double w, double h, size_t id,
	color_t c, pos_t pos);
void racg_pop(acg_base_t &base, pos_t pos);

// general insert/erase
void racg_insert(acg_base_t &base, size_t where,
	double w, double h, size_t id);
void racg_erase(acg_base_t &base, size_t where);

// all the perturbations here
// switch edge type between red/black
// 1 adjacent edge/class 1
// 2 last non-alter edge/last class 2
// 3 first alter edge/class 3
void racg_switch_adjacent(acg_base_t &base, size_t where, pos_t pos);
void racg_switch_last(acg_base_t &base, size_t where, pos_t pos);
void racg_switch_alter(acg_base_t &base, size_t where, pos_t pos);

// Reduced ACG <=> ACG
typedef std::vector<edge_t *> racg_acg_context_t;
void racg_to_acg(acg_base_t &base, racg_acg_context_t &ctx);
void acg_to_racg(acg_base_t &base, racg_acg_context_t &ctx);

} // namespace acg

#endif // ACG_RACG_H
