/*
 * Copyright (c) 2004-2005 Northwestern University.
 *                         NuCAD Group.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

 /*
 * annealing.h - SA/iterative improvement interface.
 *
 * Authors: Hai Zhou, haizhou@ece.northwestern.edu
 *          Jia Wang, jwa112@ece.northwestern.edu
 */

#ifndef ANNEALING_H
#define ANNEALING_H

#include "cost.h"

namespace fplan
{

namespace detail
{
// you can throw exception to terminate the process

// interface for perturbations
struct perturb_i
{
	// allow temperature aware perturbations
	virtual void perturbation(double T) = 0;
	virtual void reject() = 0;

	virtual void record_best() = 0;
	virtual void swap_best() = 0;

	virtual void release() = 0;

protected:
	~perturb_i() {} // no delete

}; // struct perturb_i

} // namespace detail

typedef misc::shared_ptr<detail::perturb_i> perturb_ptr;

int anneal(cost_ptr c, perturb_ptr pt, int N, FILE *fp = 0, int k = 10,
	double r = 0.90, double term_T = 0.1, double max_reject_rate = 0.90);

int greedy(cost_ptr c, perturb_ptr pt, int N, FILE *fp, double max_reject_rate = 0.9);

int greedy_time(cost_ptr c, perturb_ptr pt, int N, FILE *fp, double sec);

} // namespace fplan

#endif // ANNEALING_H
