/*
 * Copyright (c) 2004-2005 Northwestern University.
 *                         NuCAD Group.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

/*
 * cost.h - Expression based cost calculation.
 *
 * Authors: Hai Zhou, haizhou@ece.northwestern.edu
 *          Jia Wang, jwa112@ece.northwestern.edu
 *
 */

#ifndef COST_H
#define COST_H

#include "../misc/shared_ptr.h"

namespace fplan
{

namespace detail
{
	struct cost_i
	{
		virtual double cost() = 0;
		virtual void release() = 0;

	protected:
		~cost_i() {} // don't try to delete me
	}; // struct cost_i
} // namespace detail

typedef misc::shared_ptr<detail::cost_i> cost_ptr;

} // namespace fplan

// expressions need to be put into misc to allow argument dependent lookup :(
namespace misc
{

// unary
fplan::cost_ptr double_cost(double d);
fplan::cost_ptr sqrt(const fplan::cost_ptr &c);

// binary
fplan::cost_ptr pow(double d, const fplan::cost_ptr &c);
fplan::cost_ptr pow(const fplan::cost_ptr &c, double d);
fplan::cost_ptr pow(const fplan::cost_ptr &c, const fplan::cost_ptr &cc);

fplan::cost_ptr operator + (double d, const fplan::cost_ptr &c);
fplan::cost_ptr operator - (double d, const fplan::cost_ptr &c);
fplan::cost_ptr operator * (double d, const fplan::cost_ptr &c);
fplan::cost_ptr operator / (double d, const fplan::cost_ptr &c);
fplan::cost_ptr operator + (const fplan::cost_ptr &c, double d);
fplan::cost_ptr operator - (const fplan::cost_ptr &c, double d);
fplan::cost_ptr operator * (const fplan::cost_ptr &c, double d);
fplan::cost_ptr operator / (const fplan::cost_ptr &c, double d);
fplan::cost_ptr operator + (const fplan::cost_ptr &c, const fplan::cost_ptr &cc);
fplan::cost_ptr operator - (const fplan::cost_ptr &c, const fplan::cost_ptr &cc);
fplan::cost_ptr operator * (const fplan::cost_ptr &c, const fplan::cost_ptr &cc);
fplan::cost_ptr operator / (const fplan::cost_ptr &c, const fplan::cost_ptr &cc);

} // namespace misc

#endif // COST_H
