/*
 * Copyright (c) 2004-2005 Northwestern University.
 *                         NuCAD Group.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

/*
 * fplan_acg.h - Interface for ACG fploorplanning.
 *
 * Authors: Hai Zhou, haizhou@ece.northwestern.edu
 *          Jia Wang, jwa112@ece.northwestern.edu
 *
 */

#ifndef FPLAN_ACG_H
#define FPLAN_ACG_H

#include "fplan_rcg.h"
#include "../acg/acg_racg.h"

namespace fplan
{

class acg_fp_t : protected rcg_fp_t
{
protected:
	friend class acg_perturb_t;

	acg::racg_acg_context_t ctx_;
	bool reduced_;

	void require_acg();
	void require_racg();

	virtual void do_packing();
	virtual void compute_bb();

public:
	acg_fp_t(size_t n, misc::dbl_pair shapes[], size_t ids[] = 0, size_t rotated[] = 0);
	~acg_fp_t();

	void swap(acg_fp_t &r);

	acg_fp_t(acg_fp_t &r);
	acg_fp_t &operator = (acg_fp_t r);

	using rcg_fp_t::get_num_modules;
	using rcg_fp_t::get_shape;
	using rcg_fp_t::get_centers;
	using rcg_fp_t::get_physical_info;

	using rcg_fp_t::get_fplan_interface;

	using rcg_fp_t::dump;

	perturb_ptr create_perturbation(double rotateP, double exchangeP);

}; // class acg_fp_t

} // namespace fplan

namespace std
{

template<> inline void swap(fplan::acg_fp_t &a, fplan::acg_fp_t &b) {a.swap(b);}

} // namespace std

#endif // FPLAN_ACG_H
