/*
 * Copyright (c) 2004-2005 Northwestern University.
 *                         NuCAD Group.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

/*
 * fplan_rcg.h - Interface for RCG fploorplanning.
 *
 * Authors: Hai Zhou, haizhou@ece.northwestern.edu
 *          Jia Wang, jwa112@ece.northwestern.edu
 *
 */

#ifndef FPLAN_RCG_H
#define FPLAN_RCG_H

#include "annealing.h"
#include "fplan.h"
#include "../acg/acg_base.h"

namespace fplan
{

// Use the get_fplan_interface function
// instead of a public inheritance here
// since probably we want to inherit the
// implementation of rcg_fp_t.
class rcg_fp_t : protected fplan_i
{
protected:
	friend class rcg_perturb_t;

	acg::acg_base_t acg_base_;
	std::vector<size_t> rotated_;

	misc::dbl_pair shape_;
	bool mapped_;

	virtual void do_packing();
	virtual void compute_bb();

	// by pass the build process so derived classes could work
	rcg_fp_t();

public:
	rcg_fp_t(size_t n, misc::dbl_pair shapes[], size_t ids[] = 0,
		size_t rotated[] = 0, bool build_rcg = true); // last param is for derived class
	~rcg_fp_t();

	void swap(rcg_fp_t &r);

	// default ones work here
	// rcg_fp_t(const rcg_fp_t &r);
	// rcg_fp_t &operator = (rcg_fp_t r);

	virtual size_t get_num_modules();
	virtual misc::dbl_pair get_shape();
	virtual misc::dbl_pair get_centers(size_t ids[], misc::dbl_pair centers[]);
	virtual misc::dbl_pair get_physical_info(
		size_t ids[], misc::dbl_pair centers[],
		misc::dbl_pair lls[], misc::dbl_pair urs[],
		size_t rotated[], misc::dbl_pair rotated_shapes[]);

	fplan_i *get_fplan_interface() {return this;}

	bool dump(const std::string &name, misc::cstr_t module_names[] = 0);

	perturb_ptr create_perturbation(double rotateP, double exchangeP);

}; // class rcg_fp_t

} // namespace fplan

namespace std
{

template<> inline void swap(fplan::rcg_fp_t &a, fplan::rcg_fp_t &b) {a.swap(b);}

} // namespace std

#endif // FPLAN_RCG_H
