/*
 * Copyright (c) 2004-2005 Northwestern University.
 *                         NuCAD Group.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

/*
 * malloc_pool.h - Memory pool built on malloc/free
 *
 * Authors: Hai Zhou, haizhou@ece.northwestern.edu
 *          Jia Wang, jwa112@ece.northwestern.edu
 */

#ifndef MALLOC_POOL_H
#define MALLOC_POOL_H

namespace misc
{

template <size_t n>
class malloc_pool
{
	void *pool_;

	static void *&next(void *p)
	{
		return *reinterpret_cast<void **>(p);
	}

public:
	malloc_pool() : pool_(0)
	{
		// compile time check
		// make sure the pool works
		char n_is_not_big_enough[(n < sizeof(void *))? 0: 1];
		(void)n_is_not_big_enough;
	}
	
	~malloc_pool()
	{
		for (; pool_ != 0;)
		{
			void *tmp = next(pool_);
			::free(pool_);
			pool_ = tmp;
		}

		// most of time a pool is a singleton
		// clear pool_ in case a singleton is resurrected
		pool_ = 0;
	}

	void free(void *p)
	{
		next(p) = pool_;
		pool_ = p;
	}

	void *alloc()
	{
		if (pool_ == 0)
			return malloc(n);
		else
		{
			void *tmp = pool_;
			pool_ = next(pool_);
			return tmp;
		}
	}
}; // class malloc_pool<n>

} // namespace misc

#endif // MALLOC_POOL_H
