/*
 * Copyright (c) 2004-2005 Northwestern University.
 *                         NuCAD Group.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

/*
 * token_parser.cpp - Implementation of token parser and helper functions.
 *
 * Authors: Jia Wang, jwa112@ece.northwestern.edu
 *
 */

#include "common.h"

namespace misc
{

char *skip_space(char *buf, cstr_t delimit)
{
	for (; *buf != 0; ++buf)
	{
		size_t i;
		for (i = 0; delimit[i] != 0; ++i)
			if (*buf == delimit[i])
				break;
		if (delimit[i] == 0)
			return buf;
	}
	return buf;
}

char *goto_space(char *buf, cstr_t delimit)
{
	for (; *buf != 0; ++buf)
		for (size_t i = 0; delimit[i] != 0; ++i)
			if (*buf == delimit[i])
				return buf;
	return buf;
}

void split_tokens(char *buf, std::vector<cstr_t> *ptokens, cstr_t delimit)
{
	for (char *begin = buf; *begin != 0;)
	{
		char *end = goto_space(begin, delimit);
		ptokens->push_back(begin);
		if (*end == 0)
			return;
		*end = 0;
		begin = skip_space(end+1, delimit);
	}
}

// split to string vector
void split_tokens(char *buf, std::vector<std::string> *ptokens, cstr_t delimit)
{
	for (char *begin = buf; *begin != 0;)
	{
		char *end = goto_space(begin, delimit);
		ptokens->push_back(std::string(begin, end));
		if (*end == 0)
			return;
		begin = skip_space(end+1, delimit);
	}
}

// string table
typedef std::map<std::string, size_t> map_str2id;
typedef std::vector<const char *> str_v;

static map_str2id st_str_tbl;
static str_v st_strs;

size_t string_table_get_id(const char *str)
{
	map_str2id::iterator i = st_str_tbl.find(str);
	if (i == st_str_tbl.end())
	{
		i = st_str_tbl.insert(std::make_pair(str, st_strs.size())).first;
		st_strs.push_back(i->first.c_str());
	}
	return i->second;
}

const char *string_table_get_str(size_t id)
{
	assert(id < st_strs.size());

	return st_strs[id];
}

} // namespace misc
